@@ -4,30 +4,41 @@
 #
 # SPDX-License-Identifier: GPL-3.0-only
 
+import sys
 import datetime
 import random
+from typing import Union
 import unittest
 
+import adafruit_datetime
+
 import wwvb
 import uwwvb
 
+EitherDatetimeOrNone = Union[None, datetime.datetime, adafruit_datetime.datetime]
+
 
 class WWVBRoundtrip(unittest.TestCase):
     """tests of uwwvb.py"""
 
-    def assertDateTimeEqualExceptTzInfo(self, a, b):  # pylint: disable=invalid-name
+    def assertDateTimeEqualExceptTzInfo(  # pylint: disable=invalid-name
+        self, a: EitherDatetimeOrNone, b: EitherDatetimeOrNone
+    ) -> None:
         """Test two datetime objects for equality, excluding tzinfo, and allowing adafruit_datetime and core datetime modules to compare equal"""
+        assert a
+        assert b
         self.assertEqual(
             (a.year, a.month, a.day, a.hour, a.minute, a.second, a.microsecond),
             (b.year, b.month, b.day, b.hour, b.minute, b.second, b.microsecond),
         )
 
-    def test_decode(self):
+    def test_decode(self) -> None:
         """Test decoding of some minutes including a leap second.
         Each minute must decode and match the primary decoder."""
         minute = wwvb.WWVBMinuteIERS.from_datetime(
             datetime.datetime(2012, 6, 30, 23, 50)
         )
+        assert minute
         decoder = uwwvb.WWVBDecoder()
         decoder.update(uwwvb.MARK)
         any_leap_second = False
@@ -38,7 +49,7 @@ class WWVBRoundtrip(unittest.TestCase):
                 any_leap_second = True
             for code in timecode.am:
                 decoded = uwwvb.decode_wwvb(decoder.update(int(code))) or decoded
-            self.assertIsNotNone(decoded)
+            assert decoded
             self.assertDateTimeEqualExceptTzInfo(
                 minute.as_datetime_utc(),
                 uwwvb.as_datetime_utc(decoded),
@@ -46,49 +57,54 @@ class WWVBRoundtrip(unittest.TestCase):
             minute = minute.next_minute()
         self.assertTrue(any_leap_second)
 
-    def test_roundtrip(self):
+    def test_roundtrip(self) -> None:
         """Test that some big range of times all decode the same as the primary decoder"""
         dt = datetime.datetime(2002, 1, 1, 0, 0)
+        delta = datetime.timedelta(
+            minutes=7182 if sys.implementation.name == "cpython" else 86400 - 7182
+        )
         while dt.year < 2013:
             minute = wwvb.WWVBMinuteIERS.from_datetime(dt)
-            decoded = uwwvb.as_datetime_utc(uwwvb.decode_wwvb(minute.as_timecode().am))
+            assert minute
+            decoded = uwwvb.decode_wwvb([int(i) for i in minute.as_timecode().am])
+            assert decoded
             self.assertDateTimeEqualExceptTzInfo(
-                minute.as_datetime_utc(),
-                decoded,
+                minute.as_datetime_utc(), uwwvb.as_datetime_utc(decoded)
             )
-            dt = dt + datetime.timedelta(minutes=7182)
+            dt = dt + delta
 
-    def test_dst(self):
+    def test_dst(self) -> None:
         """Test of DST as handled by the small decoder"""
         for dt in (
             datetime.datetime(2021, 3, 14, 8, 59),
             datetime.datetime(2021, 3, 14, 9, 00),
             datetime.datetime(2021, 3, 14, 9, 1),
+            datetime.datetime(2021, 3, 15, 8, 59),
+            datetime.datetime(2021, 3, 15, 9, 00),
+            datetime.datetime(2021, 3, 15, 9, 1),
             datetime.datetime(2021, 11, 7, 8, 59),
             datetime.datetime(2021, 11, 7, 9, 00),
             datetime.datetime(2021, 11, 7, 9, 1),
-            datetime.datetime(2021, 12, 7, 9, 1),
+            datetime.datetime(2021, 11, 8, 8, 59),
+            datetime.datetime(2021, 11, 8, 9, 00),
+            datetime.datetime(2021, 11, 8, 9, 1),
             datetime.datetime(2021, 7, 7, 9, 1),
         ):
             minute = wwvb.WWVBMinuteIERS.from_datetime(dt)
-            decoded = uwwvb.as_datetime_local(
-                uwwvb.decode_wwvb(minute.as_timecode().am)
-            )
+            decoded = uwwvb.decode_wwvb([int(i) for i in minute.as_timecode().am])
+            assert decoded
             self.assertDateTimeEqualExceptTzInfo(
-                minute.as_datetime_local(),
-                decoded,
+                minute.as_datetime_local(), uwwvb.as_datetime_local(decoded)
             )
 
-            decoded = uwwvb.as_datetime_local(
-                uwwvb.decode_wwvb(minute.as_timecode().am),
-                dst_observed=False,
-            )
+            decoded = uwwvb.decode_wwvb([int(i) for i in minute.as_timecode().am])
+            assert decoded
             self.assertDateTimeEqualExceptTzInfo(
                 minute.as_datetime_local(dst_observed=False),
-                decoded,
+                uwwvb.as_datetime_local(decoded, dst_observed=False),
             )
 
-    def test_noise(self):
+    def test_noise(self) -> None:
         """Test of the state-machine decoder when faced with pseudorandom noise"""
         minute = wwvb.WWVBMinuteIERS.from_datetime(
             datetime.datetime(2012, 6, 30, 23, 50)
@@ -111,33 +127,34 @@ class WWVBRoundtrip(unittest.TestCase):
             decoded = decoder.update(code)
             self.assertIsNone(decoded)
         minute_maybe = decoder.update(wwvb.AmplitudeModulation.MARK)
-        self.assertIsNotNone(minute_maybe)
-        decoded = uwwvb.decode_wwvb(minute_maybe)
+        assert minute_maybe
+        decoded_minute = uwwvb.decode_wwvb(minute_maybe)
+        assert decoded_minute
         self.assertDateTimeEqualExceptTzInfo(
             minute.as_datetime_utc(),
-            uwwvb.as_datetime_utc(decoded),
+            uwwvb.as_datetime_utc(decoded_minute),
         )
         self.assertDateTimeEqualExceptTzInfo(
             minute.as_datetime_local(),
-            uwwvb.as_datetime_local(decoded),
+            uwwvb.as_datetime_local(decoded_minute),
         )
 
-    def test_noise2(self):
+    def test_noise2(self) -> None:
         """Test of the full minute decoder with targeted errors to get full coverage"""
         minute = wwvb.WWVBMinuteIERS.from_datetime(
             datetime.datetime(2012, 6, 30, 23, 50)
         )
         timecode = minute.as_timecode()
-        decoded = uwwvb.decode_wwvb(timecode.am)
+        decoded = uwwvb.decode_wwvb([int(i) for i in timecode.am])
         self.assertIsNotNone(decoded)
         for position in uwwvb.always_mark:
-            test_input = timecode.am[:]
+            test_input = [int(i) for i in timecode.am]
             for noise in (0, 1):
                 test_input[position] = noise
                 decoded = uwwvb.decode_wwvb(test_input)
                 self.assertIsNone(decoded)
         for position in uwwvb.always_zero:
-            test_input = timecode.am[:]
+            test_input = [int(i) for i in timecode.am]
             for noise in (1, 2):
                 test_input[position] = noise
                 decoded = uwwvb.decode_wwvb(test_input)
@@ -145,21 +162,21 @@ class WWVBRoundtrip(unittest.TestCase):
         for i in range(8):
             if i in (0b101, 0b010):  # Test the 6 impossible bit-combos
                 continue
-            test_input = timecode.am[:]
+            test_input = [int(i) for i in timecode.am]
             test_input[36] = i & 1
             test_input[37] = (i >> 1) & 1
             test_input[38] = (i >> 2) & 1
             decoded = uwwvb.decode_wwvb(test_input)
             self.assertIsNone(decoded)
         # Invalid year-day
-        test_input = timecode.am[:]
+        test_input = [int(i) for i in timecode.am]
         test_input[22] = 1
         test_input[23] = 1
         test_input[25] = 1
         decoded = uwwvb.decode_wwvb(test_input)
         self.assertIsNone(decoded)
 
-    def test_noise3(self):
+    def test_noise3(self) -> None:
         """Test impossible BCD values"""
         minute = wwvb.WWVBMinuteIERS.from_datetime(
             datetime.datetime(2012, 6, 30, 23, 50)
@@ -183,17 +200,18 @@ class WWVBRoundtrip(unittest.TestCase):
                 decoded = uwwvb.decode_wwvb(test_input)
                 self.assertIsNone(decoded)
 
-    def test_str(self):
+    def test_str(self) -> None:
         """Test the str() of a WWVBDecoder"""
         self.assertEqual(str(uwwvb.WWVBDecoder()), "<WWVBDecoder 1 []>")
 
-    def test_near_year_bug(self):
+    def test_near_year_bug(self) -> None:
         """Chris's WWVB software had a bug where the hours after UTC
         midnight on 12-31 of a leap year would be shown incorrectly. Check that we
         don't have that bug."""
         minute = wwvb.WWVBMinuteIERS.from_datetime(datetime.datetime(2021, 1, 1, 0, 0))
         timecode = minute.as_timecode()
-        decoded = uwwvb.decode_wwvb(timecode.am)
+        decoded = uwwvb.decode_wwvb([int(i) for i in timecode.am])
+        assert decoded
         self.assertDateTimeEqualExceptTzInfo(
             datetime.datetime(2020, 12, 31, 17, 00),  # Mountain time!
             uwwvb.as_datetime_local(decoded),
